@@ -6,13 +6,14 @@
 #include "DownloadEnum.h"
 #include "Logger.h"
 
-static std::map<int, std::string> categories;
+static std::map<DownloadEnum::Category, std::string> categories;
 
 static void initCategories()
 {
-	if (!categories.empty()) { // not initialized yet
+	if (!categories.empty()) {
 		return;
 	}
+
 	categories[DownloadEnum::CAT_MAP] = "map";
 	categories[DownloadEnum::CAT_GAME] = "game";
 	categories[DownloadEnum::CAT_SPRINGLOBBY] = "springlobby";
@@ -26,20 +27,24 @@ static void initCategories()
 	categories[DownloadEnum::CAT_COUNT] = "count";
 }
 
-const std::string DownloadEnum::getCat(DownloadEnum::Category cat)
+namespace DownloadEnum {
+
+std::string getCat(Category cat)
 {
 	initCategories();
 	return categories[cat];
 }
 
-DownloadEnum::Category DownloadEnum::getCatFromStr(const std::string& str)
+Category getCatFromStr(const std::string& str)
 {
 	initCategories();
 	for (const auto& myPair : categories) {
 		if (myPair.second == str) {
-			return (DownloadEnum::Category)myPair.first;
+			return myPair.first;
 		}
 	}
 	LOG_ERROR("Unknown category: %s", str.c_str());
-	return DownloadEnum::CAT_NONE;
+	return CAT_NONE;
 }
+
+}
\ No newline at end of file
