@@ -8,7 +8,7 @@
 #include <sstream>
 #include <stdlib.h>
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <winsock2.h>
 #else
 #include <sys/select.h>
