@@ -1,8 +1,8 @@
 #include "clar_libgit2.h"
 #include "buffer.h"
 #include "buf_text.h"
-#include "hashsig.h"
-#include "fileops.h"
+#include "git2/sys/hashsig.h"
+#include "futils.h"
 
 #define TESTSTR "Have you seen that? Have you seeeen that??"
 const char *test_string = TESTSTR;
@@ -28,7 +28,7 @@ void test_core_buffer__0(void)
 	cl_assert(git_buf_oom(&buf) == 0);
 	cl_assert_equal_s(test_string_x2, git_buf_cstr(&buf));
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 }
 
 /* test git_buf_printf */
@@ -44,7 +44,7 @@ void test_core_buffer__1(void)
 	cl_assert(git_buf_oom(&buf) == 0);
 	cl_assert_equal_s("shoop da 23 woop 42", git_buf_cstr(&buf));
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 }
 
 /* more thorough test of concatenation options */
@@ -57,7 +57,7 @@ void test_core_buffer__2(void)
 	cl_assert(buf.size == 0);
 
 	/* this must be safe to do */
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 	cl_assert(buf.size == 0);
 	cl_assert(buf.asize == 0);
 
@@ -67,7 +67,7 @@ void test_core_buffer__2(void)
 	/* cl_assert(buf.asize == 0); -- should not assume what git_buf does */
 
 	/* free should set us back to the beginning */
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 	cl_assert(buf.size == 0);
 	cl_assert(buf.asize == 0);
 
@@ -88,7 +88,7 @@ void test_core_buffer__2(void)
 	}
 	cl_assert_equal_s("++++++++++++++++++", git_buf_cstr(&buf));
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 
 	/* add data */
 	git_buf_put(&buf, "xo", 2);
@@ -108,7 +108,7 @@ void test_core_buffer__2(void)
 	cl_assert_equal_s("xoxoxoxoxoxoxoxoxoxoxoxoxoxoxoxoxoxo",
 					   git_buf_cstr(&buf));
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 
 	/* set to string */
 	git_buf_sets(&buf, test_string);
@@ -129,7 +129,7 @@ void test_core_buffer__2(void)
 	git_buf_clear(&buf);
 	cl_assert_equal_s("", git_buf_cstr(&buf));
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 
 	/* test extracting data into buffer */
 	git_buf_puts(&buf, REP4("0123456789"));
@@ -153,7 +153,7 @@ void test_core_buffer__2(void)
 	cl_assert_equal_s(REP4(REP16("x")) REP16("x") REP16("x")
 					   REP16("x") "xxxxxxxxxxxxxxx", data);
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 
 	git_buf_copy_cstr(data, sizeof(data), &buf);
 	cl_assert_equal_s("", data);
@@ -179,7 +179,7 @@ void test_core_buffer__3(void)
 	cl_assert(git_buf_oom(&buf) == 0);
 	cl_assert_equal_s(test_4096, git_buf_cstr(&buf));
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 }
 
 /* let's try some producer/consumer tests */
@@ -212,7 +212,7 @@ void test_core_buffer__4(void)
 	git_buf_consume(&buf, buf.ptr + buf.size);
 	cl_assert_equal_s("", git_buf_cstr(&buf));
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 }
 
 
@@ -231,11 +231,11 @@ check_buf_append(
 	git_buf_puts(&tgt, data_b);
 	cl_assert(git_buf_oom(&tgt) == 0);
 	cl_assert_equal_s(expected_data, git_buf_cstr(&tgt));
-	cl_assert(tgt.size == expected_size);
+	cl_assert_equal_i(tgt.size, expected_size);
 	if (expected_asize > 0)
-		cl_assert(tgt.asize == expected_asize);
+		cl_assert_equal_i(tgt.asize, expected_asize);
 
-	git_buf_free(&tgt);
+	git_buf_dispose(&tgt);
 }
 
 static void
@@ -275,17 +275,16 @@ check_buf_append_abc(
 	cl_assert(git_buf_oom(&buf) == 0);
 	cl_assert_equal_s(expected_abcabc, git_buf_cstr(&buf));
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 }
 
 /* more variations on append tests */
 void test_core_buffer__5(void)
 {
-	check_buf_append("", "", "", 0, 8);
-	check_buf_append("a", "", "a", 1, 8);
+	check_buf_append("", "", "", 0, 0);
+	check_buf_append("a", "", "a", 1, 0);
 	check_buf_append("", "a", "a", 1, 8);
 	check_buf_append("", "a", "a", 1, 8);
-	check_buf_append("a", "", "a", 1, 8);
 	check_buf_append("a", "b", "ab", 2, 8);
 	check_buf_append("", "abcdefgh", "abcdefgh", 8, 16);
 	check_buf_append("abcdefgh", "", "abcdefgh", 8, 16);
@@ -309,7 +308,7 @@ void test_core_buffer__5(void)
 					 REP16("x") REP16("o"), 32, 40);
 
 	check_buf_append(test_4096, "", test_4096, 4096, 4104);
-	check_buf_append(test_4096, test_4096, test_8192, 8192, 9240);
+	check_buf_append(test_4096, test_4096, test_8192, 8192, 8200);
 
 	/* check sequences of appends */
 	check_buf_append_abc("a", "b", "c",
@@ -341,8 +340,8 @@ void test_core_buffer__6(void)
 	cl_assert_equal_s("bar", git_buf_cstr(&a));
 	cl_assert_equal_s("foo", git_buf_cstr(&b));
 
-	git_buf_free(&a);
-	git_buf_free(&b);
+	git_buf_dispose(&a);
+	git_buf_dispose(&b);
 }
 
 
@@ -368,7 +367,7 @@ void test_core_buffer__7(void)
 	cl_assert_equal_s(NULL, b);
 	cl_assert_equal_s("", a.ptr);
 
-	git_buf_free(&a);
+	git_buf_dispose(&a);
 
 	b = git__strdup(fun);
 	git_buf_attach(&a, b, 0);
@@ -377,7 +376,7 @@ void test_core_buffer__7(void)
 	cl_assert(a.size == strlen(fun));
 	cl_assert(a.asize == strlen(fun) + 1);
 
-	git_buf_free(&a);
+	git_buf_dispose(&a);
 
 	b = git__strdup(fun);
 	git_buf_attach(&a, b, strlen(fun) + 1);
@@ -386,7 +385,7 @@ void test_core_buffer__7(void)
 	cl_assert(a.size == strlen(fun));
 	cl_assert(a.asize == strlen(fun) + 1);
 
-	git_buf_free(&a);
+	git_buf_dispose(&a);
 }
 
 
@@ -402,7 +401,24 @@ check_joinbuf_2(
 	git_buf_join(&buf, sep, a, b);
 	cl_assert(git_buf_oom(&buf) == 0);
 	cl_assert_equal_s(expected, git_buf_cstr(&buf));
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
+}
+
+static void
+check_joinbuf_overlapped(
+	const char *oldval,
+	int ofs_a,
+	const char *b,
+	const char *expected)
+{
+	char sep = '/';
+	git_buf buf = GIT_BUF_INIT;
+
+	git_buf_sets(&buf, oldval);
+	git_buf_join(&buf, sep, buf.ptr + ofs_a, b);
+	cl_assert(git_buf_oom(&buf) == 0);
+	cl_assert_equal_s(expected, git_buf_cstr(&buf));
+	git_buf_dispose(&buf);
 }
 
 static void
@@ -421,7 +437,7 @@ check_joinbuf_n_2(
 	cl_assert(git_buf_oom(&buf) == 0);
 	cl_assert_equal_s(expected, git_buf_cstr(&buf));
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 }
 
 static void
@@ -437,7 +453,7 @@ check_joinbuf_n_4(
 	git_buf_join_n(&buf, sep, 4, a, b, c, d);
 	cl_assert(git_buf_oom(&buf) == 0);
 	cl_assert_equal_s(expected, git_buf_cstr(&buf));
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 }
 
 /* test join */
@@ -457,7 +473,7 @@ void test_core_buffer__8(void)
 	cl_assert(git_buf_oom(&a) == 0);
 	cl_assert_equal_s("foo/bar/baz", git_buf_cstr(&a));
 
-	git_buf_free(&a);
+	git_buf_dispose(&a);
 
 	check_joinbuf_2(NULL, "", "");
 	check_joinbuf_2(NULL, "a", "a");
@@ -480,6 +496,20 @@ void test_core_buffer__8(void)
 	check_joinbuf_2("/abcd/", "defg/", "/abcd/defg/");
 	check_joinbuf_2("/abcd/", "/defg/", "/abcd/defg/");
 
+	check_joinbuf_overlapped("abcd", 0, "efg", "abcd/efg");
+	check_joinbuf_overlapped("abcd", 1, "efg", "bcd/efg");
+	check_joinbuf_overlapped("abcd", 2, "efg", "cd/efg");
+	check_joinbuf_overlapped("abcd", 3, "efg", "d/efg");
+	check_joinbuf_overlapped("abcd", 4, "efg", "efg");
+	check_joinbuf_overlapped("abc/", 2, "efg", "c/efg");
+	check_joinbuf_overlapped("abc/", 3, "efg", "/efg");
+	check_joinbuf_overlapped("abc/", 4, "efg", "efg");
+	check_joinbuf_overlapped("abcd", 3, "", "d/");
+	check_joinbuf_overlapped("abcd", 4, "", "");
+	check_joinbuf_overlapped("abc/", 2, "", "c/");
+	check_joinbuf_overlapped("abc/", 3, "", "/");
+	check_joinbuf_overlapped("abc/", 4, "", "");
+
 	check_joinbuf_n_2("", "", "");
 	check_joinbuf_n_2("", "a", "a");
 	check_joinbuf_n_2("", "/a", "/a");
@@ -548,7 +578,7 @@ void test_core_buffer__9(void)
 		}
 	}
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 }
 
 void test_core_buffer__10(void)
@@ -565,7 +595,39 @@ void test_core_buffer__10(void)
 	cl_git_pass(git_buf_join_n(&a, '/', 2, a.ptr, "more"));
 	cl_assert_equal_s(a.ptr, "test/string/join/test/string/join/more");
 
-	git_buf_free(&a);
+	git_buf_dispose(&a);
+}
+
+void test_core_buffer__join3(void)
+{
+	git_buf a = GIT_BUF_INIT;
+
+	cl_git_pass(git_buf_join3(&a, '/', "test", "string", "join"));
+	cl_assert_equal_s("test/string/join", a.ptr);
+	cl_git_pass(git_buf_join3(&a, '/', "test/", "string", "join"));
+	cl_assert_equal_s("test/string/join", a.ptr);
+	cl_git_pass(git_buf_join3(&a, '/', "test/", "/string", "join"));
+	cl_assert_equal_s("test/string/join", a.ptr);
+	cl_git_pass(git_buf_join3(&a, '/', "test/", "/string/", "join"));
+	cl_assert_equal_s("test/string/join", a.ptr);
+	cl_git_pass(git_buf_join3(&a, '/', "test/", "/string/", "/join"));
+	cl_assert_equal_s("test/string/join", a.ptr);
+
+	cl_git_pass(git_buf_join3(&a, '/', "", "string", "join"));
+	cl_assert_equal_s("string/join", a.ptr);
+	cl_git_pass(git_buf_join3(&a, '/', "", "string/", "join"));
+	cl_assert_equal_s("string/join", a.ptr);
+	cl_git_pass(git_buf_join3(&a, '/', "", "string/", "/join"));
+	cl_assert_equal_s("string/join", a.ptr);
+
+	cl_git_pass(git_buf_join3(&a, '/', "string", "", "join"));
+	cl_assert_equal_s("string/join", a.ptr);
+	cl_git_pass(git_buf_join3(&a, '/', "string/", "", "join"));
+	cl_assert_equal_s("string/join", a.ptr);
+	cl_git_pass(git_buf_join3(&a, '/', "string/", "", "/join"));
+	cl_assert_equal_s("string/join", a.ptr);
+
+	git_buf_dispose(&a);
 }
 
 void test_core_buffer__11(void)
@@ -615,7 +677,7 @@ void test_core_buffer__11(void)
 	cl_git_pass(git_buf_text_common_prefix(&a, &t));
 	cl_assert_equal_s(a.ptr, "");
 
-	git_buf_free(&a);
+	git_buf_dispose(&a);
 }
 
 void test_core_buffer__rfind_variants(void)
@@ -639,7 +701,7 @@ void test_core_buffer__rfind_variants(void)
 	cl_assert(git_buf_rfind(&a, 'q') == -1);
 	cl_assert(git_buf_rfind_next(&a, 'q') == -1);
 
-	git_buf_free(&a);
+	git_buf_dispose(&a);
 }
 
 void test_core_buffer__puts_escaped(void)
@@ -662,7 +724,7 @@ void test_core_buffer__puts_escaped(void)
 	cl_git_pass(git_buf_text_puts_escape_regex(&a, "^match\\s*[A-Z]+.*"));
 	cl_assert_equal_s("\\^match\\\\s\\*\\[A-Z\\]\\+\\.\\*", a.ptr);
 
-	git_buf_free(&a);
+	git_buf_dispose(&a);
 }
 
 static void assert_unescape(char *expected, char *to_unescape) {
@@ -673,7 +735,7 @@ static void assert_unescape(char *expected, char *to_unescape) {
 	cl_assert_equal_s(expected, buf.ptr);
 	cl_assert_equal_sz(strlen(expected), buf.size);
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 }
 
 void test_core_buffer__unescape(void)
@@ -685,7 +747,7 @@ void test_core_buffer__unescape(void)
 	assert_unescape("", "");
 }
 
-void test_core_buffer__base64(void)
+void test_core_buffer__encode_base64(void)
 {
 	git_buf buf = GIT_BUF_INIT;
 
@@ -696,18 +758,97 @@ void test_core_buffer__base64(void)
 	 * 0x 1d 06 21 29 1c 30
 	 *     d  G  h  p  c  w
 	 */
-	cl_git_pass(git_buf_put_base64(&buf, "this", 4));
+	cl_git_pass(git_buf_encode_base64(&buf, "this", 4));
 	cl_assert_equal_s("dGhpcw==", buf.ptr);
 
 	git_buf_clear(&buf);
-	cl_git_pass(git_buf_put_base64(&buf, "this!", 5));
+	cl_git_pass(git_buf_encode_base64(&buf, "this!", 5));
 	cl_assert_equal_s("dGhpcyE=", buf.ptr);
 
 	git_buf_clear(&buf);
-	cl_git_pass(git_buf_put_base64(&buf, "this!\n", 6));
+	cl_git_pass(git_buf_encode_base64(&buf, "this!\n", 6));
 	cl_assert_equal_s("dGhpcyEK", buf.ptr);
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
+}
+
+void test_core_buffer__decode_base64(void)
+{
+	git_buf buf = GIT_BUF_INIT;
+
+	cl_git_pass(git_buf_decode_base64(&buf, "dGhpcw==", 8));
+	cl_assert_equal_s("this", buf.ptr);
+
+	git_buf_clear(&buf);
+	cl_git_pass(git_buf_decode_base64(&buf, "dGhpcyE=", 8));
+	cl_assert_equal_s("this!", buf.ptr);
+
+	git_buf_clear(&buf);
+	cl_git_pass(git_buf_decode_base64(&buf, "dGhpcyEK", 8));
+	cl_assert_equal_s("this!\n", buf.ptr);
+
+	cl_git_fail(git_buf_decode_base64(&buf, "This is not a valid base64 string!!!", 36));
+	cl_assert_equal_s("this!\n", buf.ptr);
+
+	git_buf_dispose(&buf);
+}
+
+void test_core_buffer__encode_base85(void)
+{
+	git_buf buf = GIT_BUF_INIT;
+
+	cl_git_pass(git_buf_encode_base85(&buf, "this", 4));
+	cl_assert_equal_s("bZBXF", buf.ptr);
+	git_buf_clear(&buf);
+
+	cl_git_pass(git_buf_encode_base85(&buf, "two rnds", 8));
+	cl_assert_equal_s("ba!tca&BaE", buf.ptr);
+	git_buf_clear(&buf);
+
+	cl_git_pass(git_buf_encode_base85(&buf, "this is base 85 encoded",
+		strlen("this is base 85 encoded")));
+	cl_assert_equal_s("bZBXFAZc?TVqtS-AUHK3Wo~0{WMyOk", buf.ptr);
+	git_buf_clear(&buf);
+
+	git_buf_dispose(&buf);
+}
+
+void test_core_buffer__decode_base85(void)
+{
+	git_buf buf = GIT_BUF_INIT;
+
+	cl_git_pass(git_buf_decode_base85(&buf, "bZBXF", 5, 4));
+	cl_assert_equal_sz(4, buf.size);
+	cl_assert_equal_s("this", buf.ptr);
+	git_buf_clear(&buf);
+
+	cl_git_pass(git_buf_decode_base85(&buf, "ba!tca&BaE", 10, 8));
+	cl_assert_equal_sz(8, buf.size);
+	cl_assert_equal_s("two rnds", buf.ptr);
+	git_buf_clear(&buf);
+
+	cl_git_pass(git_buf_decode_base85(&buf, "bZBXFAZc?TVqtS-AUHK3Wo~0{WMyOk", 30, 23));
+	cl_assert_equal_sz(23, buf.size);
+	cl_assert_equal_s("this is base 85 encoded", buf.ptr);
+	git_buf_clear(&buf);
+
+	git_buf_dispose(&buf);
+}
+
+void test_core_buffer__decode_base85_fails_gracefully(void)
+{
+	git_buf buf = GIT_BUF_INIT;
+
+	git_buf_puts(&buf, "foobar");
+
+	cl_git_fail(git_buf_decode_base85(&buf, "invalid charsZZ", 15, 42));
+	cl_git_fail(git_buf_decode_base85(&buf, "invalidchars__ ", 15, 42));
+	cl_git_fail(git_buf_decode_base85(&buf, "overflowZZ~~~~~", 15, 42));
+	cl_git_fail(git_buf_decode_base85(&buf, "truncated", 9, 42));
+	cl_assert_equal_sz(6, buf.size);
+	cl_assert_equal_s("foobar", buf.ptr);
+
+	git_buf_dispose(&buf);
 }
 
 void test_core_buffer__classify_with_utf8(void)
@@ -727,7 +868,7 @@ void test_core_buffer__classify_with_utf8(void)
 	cl_assert(!git_buf_text_contains_nul(&b));
 
 	b.ptr = data1; b.size = b.asize = data1len;
-	cl_assert(git_buf_text_is_binary(&b));
+	cl_assert(!git_buf_text_is_binary(&b));
 	cl_assert(!git_buf_text_contains_nul(&b));
 
 	b.ptr = data2; b.size = b.asize = data2len;
@@ -826,7 +967,7 @@ void test_core_buffer__similarity_metric(void)
 	cl_git_pass(git_buf_sets(&buf, SIMILARITY_TEST_DATA_1));
 	cl_git_pass(git_hashsig_create(&a, buf.ptr, buf.size, GIT_HASHSIG_NORMAL));
 
-	cl_git_pass(git_futils_mkdir("scratch", NULL, 0755, GIT_MKDIR_PATH));
+	cl_git_pass(git_futils_mkdir("scratch", 0755, GIT_MKDIR_PATH));
 	cl_git_mkfile("scratch/testdata", SIMILARITY_TEST_DATA_1);
 	cl_git_pass(git_hashsig_create_fromfile(
 		&b, "scratch/testdata", GIT_HASHSIG_NORMAL));
@@ -836,7 +977,7 @@ void test_core_buffer__similarity_metric(void)
 	git_hashsig_free(a);
 	git_hashsig_free(b);
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 	git_futils_rmdir_r("scratch", NULL, GIT_RMDIR_REMOVE_FILES);
 }
 
@@ -916,7 +1057,7 @@ void test_core_buffer__similarity_metric_whitespace(void)
 		}
 	}
 
-	git_buf_free(&buf);
+	git_buf_dispose(&buf);
 }
 
 #include "../filter/crlf.h"
@@ -953,7 +1094,8 @@ void test_core_buffer__lf_and_crlf_conversions(void)
 
 	cl_git_pass(git_buf_text_lf_to_crlf(&tgt, &src));
 	check_buf("crlf\r\ncrlf\r\ncrlf\r\ncrlf\r\n", tgt);
-	check_buf(src.ptr, tgt);
+
+	git_buf_sets(&src, "crlf\r\ncrlf\r\ncrlf\r\ncrlf\r\n");
 
 	cl_git_pass(git_buf_text_crlf_to_lf(&tgt, &src));
 	check_buf("crlf\ncrlf\ncrlf\ncrlf\n", tgt);
@@ -962,7 +1104,8 @@ void test_core_buffer__lf_and_crlf_conversions(void)
 
 	cl_git_pass(git_buf_text_lf_to_crlf(&tgt, &src));
 	check_buf("\r\ncrlf\r\ncrlf\r\ncrlf\r\ncrlf\r\ncrlf", tgt);
-	check_buf(src.ptr, tgt);
+
+	git_buf_sets(&src, "\r\ncrlf\r\ncrlf\r\ncrlf\r\ncrlf\r\ncrlf");
 
 	cl_git_pass(git_buf_text_crlf_to_lf(&tgt, &src));
 	check_buf("\ncrlf\ncrlf\ncrlf\ncrlf\ncrlf", tgt);
@@ -973,6 +1116,9 @@ void test_core_buffer__lf_and_crlf_conversions(void)
 
 	cl_git_pass(git_buf_text_lf_to_crlf(&tgt, &src));
 	check_buf("\r\nlf\r\nlf\r\ncrlf\r\nlf\r\nlf\r\ncrlf\r\n", tgt);
+
+	git_buf_sets(&src, "\nlf\nlf\ncrlf\r\nlf\nlf\ncrlf\r\n");
+
 	cl_git_pass(git_buf_text_crlf_to_lf(&tgt, &src));
 	check_buf("\nlf\nlf\ncrlf\nlf\nlf\ncrlf\n", tgt);
 
@@ -982,6 +1128,7 @@ void test_core_buffer__lf_and_crlf_conversions(void)
 
 	cl_git_pass(git_buf_text_lf_to_crlf(&tgt, &src));
 	check_buf("\r\ncrlf\r\ncrlf\r\nlf\r\ncrlf\r\ncrlf", tgt);
+
 	cl_git_pass(git_buf_text_crlf_to_lf(&tgt, &src));
 	check_buf("\ncrlf\ncrlf\nlf\ncrlf\ncrlf", tgt);
 
@@ -991,6 +1138,9 @@ void test_core_buffer__lf_and_crlf_conversions(void)
 
 	cl_git_pass(git_buf_text_lf_to_crlf(&tgt, &src));
 	check_buf("\rcrlf\r\nlf\r\nlf\r\ncr\rcrlf\r\nlf\r\ncr\r", tgt);
+
+	git_buf_sets(&src, "\rcrlf\r\nlf\nlf\ncr\rcrlf\r\nlf\ncr\r");
+
 	cl_git_pass(git_buf_text_crlf_to_lf(&tgt, &src));
 	check_buf("\rcrlf\nlf\nlf\ncr\rcrlf\nlf\ncr\r", tgt);
 
@@ -1000,40 +1150,93 @@ void test_core_buffer__lf_and_crlf_conversions(void)
 	cl_git_pass(git_buf_text_crlf_to_lf(&tgt, &src));
 	check_buf("\rcr\r", tgt);
 
-	git_buf_free(&src);
-	git_buf_free(&tgt);
+	git_buf_dispose(&src);
+	git_buf_dispose(&tgt);
 
 	/* blob correspondence tests */
 
 	git_buf_sets(&src, ALL_CRLF_TEXT_RAW);
 	cl_git_pass(git_buf_text_lf_to_crlf(&tgt, &src));
 	check_buf(ALL_CRLF_TEXT_AS_CRLF, tgt);
+	git_buf_sets(&src, ALL_CRLF_TEXT_RAW);
 	cl_git_pass(git_buf_text_crlf_to_lf(&tgt, &src));
 	check_buf(ALL_CRLF_TEXT_AS_LF, tgt);
-	git_buf_free(&src);
-	git_buf_free(&tgt);
+	git_buf_dispose(&src);
+	git_buf_dispose(&tgt);
 
 	git_buf_sets(&src, ALL_LF_TEXT_RAW);
 	cl_git_pass(git_buf_text_lf_to_crlf(&tgt, &src));
 	check_buf(ALL_LF_TEXT_AS_CRLF, tgt);
+	git_buf_sets(&src, ALL_LF_TEXT_RAW);
 	cl_git_pass(git_buf_text_crlf_to_lf(&tgt, &src));
 	check_buf(ALL_LF_TEXT_AS_LF, tgt);
-	git_buf_free(&src);
-	git_buf_free(&tgt);
+	git_buf_dispose(&src);
+	git_buf_dispose(&tgt);
 
 	git_buf_sets(&src, MORE_CRLF_TEXT_RAW);
 	cl_git_pass(git_buf_text_lf_to_crlf(&tgt, &src));
 	check_buf(MORE_CRLF_TEXT_AS_CRLF, tgt);
+	git_buf_sets(&src, MORE_CRLF_TEXT_RAW);
 	cl_git_pass(git_buf_text_crlf_to_lf(&tgt, &src));
 	check_buf(MORE_CRLF_TEXT_AS_LF, tgt);
-	git_buf_free(&src);
-	git_buf_free(&tgt);
+	git_buf_dispose(&src);
+	git_buf_dispose(&tgt);
 
 	git_buf_sets(&src, MORE_LF_TEXT_RAW);
 	cl_git_pass(git_buf_text_lf_to_crlf(&tgt, &src));
 	check_buf(MORE_LF_TEXT_AS_CRLF, tgt);
+	git_buf_sets(&src, MORE_LF_TEXT_RAW);
 	cl_git_pass(git_buf_text_crlf_to_lf(&tgt, &src));
 	check_buf(MORE_LF_TEXT_AS_LF, tgt);
-	git_buf_free(&src);
-	git_buf_free(&tgt);
+	git_buf_dispose(&src);
+	git_buf_dispose(&tgt);
+}
+
+void test_core_buffer__dont_grow_borrowed(void)
+{
+	const char *somestring = "blah blah";
+	git_buf buf = GIT_BUF_INIT;
+
+	git_buf_attach_notowned(&buf, somestring, strlen(somestring) + 1);
+	cl_assert_equal_p(somestring, buf.ptr);
+	cl_assert_equal_i(0, buf.asize);
+	cl_assert_equal_i(strlen(somestring) + 1, buf.size);
+
+	cl_git_fail_with(GIT_EINVALID, git_buf_grow(&buf, 1024));
+}
+
+void test_core_buffer__dont_hit_infinite_loop_when_resizing(void)
+{
+	git_buf buf = GIT_BUF_INIT;
+
+	cl_git_pass(git_buf_puts(&buf, "foobar"));
+	/*
+	 * We do not care whether this succeeds or fails, which
+	 * would depend on platform-specific allocation
+	 * semantics. We only want to know that the function
+	 * actually returns.
+	 */
+	(void)git_buf_try_grow(&buf, SIZE_MAX, true);
+
+	git_buf_dispose(&buf);
+}
+
+void test_core_buffer__avoid_printing_into_oom_buffer(void)
+{
+	git_buf buf = GIT_BUF_INIT;
+
+	/* Emulate OOM situation with a previous allocation */
+	buf.asize = 8;
+	buf.ptr = git_buf__oom;
+
+	/*
+	 * Print the same string again. As the buffer still has
+	 * an `asize` of 8 due to the previous print,
+	 * `ENSURE_SIZE` would not try to reallocate the array at
+	 * all. As it didn't explicitly check for `git_buf__oom`
+	 * in earlier versions, this would've resulted in it
+	 * returning successfully and thus `git_buf_puts` would
+	 * just print into the `git_buf__oom` array.
+	 */
+	cl_git_fail(git_buf_puts(&buf, "foobar"));
 }
