@@ -1,32 +1,40 @@
 #include "clar_libgit2.h"
-#include "fileops.h"
+#include "futils.h"
 #include "repository.h"
 #include "config.h"
 #include "path.h"
+#include "config/config_helpers.h"
+#include "repo/repo_helpers.h"
 
 enum repo_mode {
 	STANDARD_REPOSITORY = 0,
 	BARE_REPOSITORY = 1
 };
 
-static git_repository *_repo = NULL;
-static mode_t g_umask = 0;
+static git_repository *g_repo = NULL;
+static git_buf g_global_path = GIT_BUF_INIT;
 
 void test_repo_init__initialize(void)
 {
-	_repo = NULL;
+	g_repo = NULL;
 
-	/* load umask if not already loaded */
-	if (!g_umask) {
-		g_umask = p_umask(022);
-		(void)p_umask(g_umask);
-	}
+	git_libgit2_opts(GIT_OPT_GET_SEARCH_PATH, GIT_CONFIG_LEVEL_GLOBAL,
+			 &g_global_path);
+}
+
+void test_repo_init__cleanup(void)
+{
+	git_libgit2_opts(GIT_OPT_SET_SEARCH_PATH, GIT_CONFIG_LEVEL_GLOBAL,
+		g_global_path.ptr);
+	git_buf_dispose(&g_global_path);
+
+	cl_fixture_cleanup("tmp_global_path");
 }
 
 static void cleanup_repository(void *path)
 {
-	git_repository_free(_repo);
-	_repo = NULL;
+	git_repository_free(g_repo);
+	g_repo = NULL;
 
 	cl_fixture_cleanup((const char *)path);
 }
@@ -41,9 +49,9 @@ static void ensure_repository_init(
 
 	cl_assert(!git_path_isdir(working_directory));
 
-	cl_git_pass(git_repository_init(&_repo, working_directory, is_bare));
+	cl_git_pass(git_repository_init(&g_repo, working_directory, is_bare));
 
-	workdir = git_repository_workdir(_repo);
+	workdir = git_repository_workdir(g_repo);
 	if (workdir != NULL || expected_working_directory != NULL) {
 		cl_assert(
 			git__suffixcmp(workdir, expected_working_directory) == 0
@@ -51,19 +59,19 @@ static void ensure_repository_init(
 	}
 
 	cl_assert(
-		git__suffixcmp(git_repository_path(_repo), expected_path_repository) == 0
+		git__suffixcmp(git_repository_path(g_repo), expected_path_repository) == 0
 	);
 
-	cl_assert(git_repository_is_bare(_repo) == is_bare);
+	cl_assert(git_repository_is_bare(g_repo) == is_bare);
 
 #ifdef GIT_WIN32
 	if (!is_bare) {
-		DWORD fattrs = GetFileAttributes(git_repository_path(_repo));
+		DWORD fattrs = GetFileAttributes(git_repository_path(g_repo));
 		cl_assert((fattrs & FILE_ATTRIBUTE_HIDDEN) != 0);
 	}
 #endif
 
-	cl_assert(git_repository_is_empty(_repo));
+	cl_assert(git_repository_is_empty(g_repo));
 }
 
 void test_repo_init__standard_repo(void)
@@ -98,24 +106,25 @@ void test_repo_init__bare_repo_escaping_current_workdir(void)
 	cl_git_pass(git_path_prettify_dir(&path_current_workdir, ".", NULL));
 
 	cl_git_pass(git_buf_joinpath(&path_repository, git_buf_cstr(&path_current_workdir), "a/b/c"));
-	cl_git_pass(git_futils_mkdir_r(git_buf_cstr(&path_repository), NULL, GIT_DIR_MODE));
+	cl_git_pass(git_futils_mkdir_r(git_buf_cstr(&path_repository), GIT_DIR_MODE));
 
 	/* Change the current working directory */
 	cl_git_pass(chdir(git_buf_cstr(&path_repository)));
 
 	/* Initialize a bare repo with a relative path escaping out of the current working directory */
-	cl_git_pass(git_repository_init(&_repo, "../d/e.git", 1));
-	cl_git_pass(git__suffixcmp(git_repository_path(_repo), "/a/b/d/e.git/"));
+	cl_git_pass(git_repository_init(&g_repo, "../d/e.git", 1));
+	cl_git_pass(git__suffixcmp(git_repository_path(g_repo), "/a/b/d/e.git/"));
 
-	git_repository_free(_repo);
+	git_repository_free(g_repo);
+	g_repo = NULL;
 
 	/* Open a bare repo with a relative path escaping out of the current working directory */
-	cl_git_pass(git_repository_open(&_repo, "../d/e.git"));
+	cl_git_pass(git_repository_open(&g_repo, "../d/e.git"));
 
 	cl_git_pass(chdir(git_buf_cstr(&path_current_workdir)));
 
-	git_buf_free(&path_current_workdir);
-	git_buf_free(&path_repository);
+	git_buf_dispose(&path_current_workdir);
+	git_buf_dispose(&path_repository);
 
 	cleanup_repository("a");
 }
@@ -125,11 +134,12 @@ void test_repo_init__reinit_bare_repo(void)
 	cl_set_cleanup(&cleanup_repository, "reinit.git");
 
 	/* Initialize the repository */
-	cl_git_pass(git_repository_init(&_repo, "reinit.git", 1));
-	git_repository_free(_repo);
+	cl_git_pass(git_repository_init(&g_repo, "reinit.git", 1));
+	git_repository_free(g_repo);
+	g_repo = NULL;
 
 	/* Reinitialize the repository */
-	cl_git_pass(git_repository_init(&_repo, "reinit.git", 1));
+	cl_git_pass(git_repository_init(&g_repo, "reinit.git", 1));
 }
 
 void test_repo_init__reinit_too_recent_bare_repo(void)
@@ -137,8 +147,8 @@ void test_repo_init__reinit_too_recent_bare_repo(void)
 	git_config *config;
 
 	/* Initialize the repository */
-	cl_git_pass(git_repository_init(&_repo, "reinit.git", 1));
-	git_repository_config(&config, _repo);
+	cl_git_pass(git_repository_init(&g_repo, "reinit.git", 1));
+	git_repository_config(&config, g_repo);
 
 	/*
 	 * Hack the config of the repository to make it look like it has
@@ -147,10 +157,11 @@ void test_repo_init__reinit_too_recent_bare_repo(void)
 	cl_git_pass(git_config_set_int32(config, "core.repositoryformatversion", 42));
 
 	git_config_free(config);
-	git_repository_free(_repo);
+	git_repository_free(g_repo);
+	g_repo = NULL;
 
 	/* Try to reinitialize the repository */
-	cl_git_fail(git_repository_init(&_repo, "reinit.git", 1));
+	cl_git_fail(git_repository_init(&g_repo, "reinit.git", 1));
 
 	cl_fixture_cleanup("reinit.git");
 }
@@ -164,19 +175,19 @@ void test_repo_init__additional_templates(void)
 	ensure_repository_init("tester", 0, "tester/.git/", "tester/");
 
 	cl_git_pass(
-		git_buf_joinpath(&path, git_repository_path(_repo), "description"));
+		git_buf_joinpath(&path, git_repository_path(g_repo), "description"));
 	cl_assert(git_path_isfile(git_buf_cstr(&path)));
 
 	cl_git_pass(
-		git_buf_joinpath(&path, git_repository_path(_repo), "info/exclude"));
+		git_buf_joinpath(&path, git_repository_path(g_repo), "info/exclude"));
 	cl_assert(git_path_isfile(git_buf_cstr(&path)));
 
 	cl_git_pass(
-		git_buf_joinpath(&path, git_repository_path(_repo), "hooks"));
+		git_buf_joinpath(&path, git_repository_path(g_repo), "hooks"));
 	cl_assert(git_path_isdir(git_buf_cstr(&path)));
 	/* won't confirm specific contents of hooks dir since it may vary */
 
-	git_buf_free(&path);
+	git_buf_dispose(&path);
 }
 
 static void assert_config_entry_on_init_bytype(
@@ -189,9 +200,9 @@ static void assert_config_entry_on_init_bytype(
 
 	cl_set_cleanup(&cleanup_repository, "config_entry");
 
-	cl_git_pass(git_repository_init(&_repo, repo_path, is_bare));
+	cl_git_pass(git_repository_init(&g_repo, repo_path, is_bare));
 
-	cl_git_pass(git_repository_config(&config, _repo));
+	cl_git_pass(git_repository_config(&config, g_repo));
 	error = git_config_get_bool(&current_value, config, config_key);
 	git_config_free(config);
 
@@ -207,7 +218,8 @@ static void assert_config_entry_on_init(
 	const char *config_key, int expected_value)
 {
 	assert_config_entry_on_init_bytype(config_key, expected_value, true);
-	git_repository_free(_repo);
+	git_repository_free(g_repo);
+	g_repo = NULL;
 
 	assert_config_entry_on_init_bytype(config_key, expected_value, false);
 }
@@ -230,6 +242,71 @@ void test_repo_init__detect_ignorecase(void)
 		"core.ignorecase", found_without_match ? true : GIT_ENOTFOUND);
 }
 
+/*
+ * Windows: if the filesystem supports symlinks (because we're running
+ * as administrator, or because the user has opted into it for normal
+ * users) then we can also opt-in explicitly by settings `core.symlinks`
+ * in the global config.  Symlinks remain off by default.
+ */
+
+void test_repo_init__symlinks_win32_enabled_by_global_config(void)
+{
+#ifndef GIT_WIN32
+	cl_skip();
+#else
+	git_config *config, *repo_config;
+	int val;
+
+	if (!git_path_supports_symlinks("link"))
+		cl_skip();
+
+	create_tmp_global_config("tmp_global_config", "core.symlinks", "true");
+
+	/*
+	 * Create a new repository (can't use `assert_config_on_init` since we
+	 * want to examine configuration levels with more granularity.)
+	 */
+	cl_git_pass(git_repository_init(&g_repo, "config_entry/test.non.bare.git", false));
+
+	/* Ensure that core.symlinks remains set (via the global config). */
+	cl_git_pass(git_repository_config(&config, g_repo));
+	cl_git_pass(git_config_get_bool(&val, config, "core.symlinks"));
+	cl_assert_equal_i(1, val);
+
+	/*
+	 * Ensure that the repository config does not set core.symlinks.
+	 * It should remain inherited.
+	 */
+	cl_git_pass(git_config_open_level(&repo_config, config, GIT_CONFIG_LEVEL_LOCAL));
+	cl_git_fail_with(GIT_ENOTFOUND, git_config_get_bool(&val, repo_config, "core.symlinks"));
+	git_config_free(repo_config);
+
+	git_config_free(config);
+
+	git_repository_free(g_repo);
+	g_repo = NULL;
+#endif
+}
+
+void test_repo_init__symlinks_win32_off_by_default(void)
+{
+#ifndef GIT_WIN32
+	cl_skip();
+#else
+	assert_config_entry_on_init("core.symlinks", false);
+#endif
+}
+
+void test_repo_init__symlinks_posix_detected(void)
+{
+#ifdef GIT_WIN32
+	cl_skip();
+#else
+	assert_config_entry_on_init(
+	    "core.symlinks", git_path_supports_symlinks("link") ? GIT_ENOTFOUND : false);
+#endif
+}
+
 void test_repo_init__detect_precompose_unicode_required(void)
 {
 #ifdef GIT_USE_ICONV
@@ -254,18 +331,18 @@ void test_repo_init__reinit_doesnot_overwrite_ignorecase(void)
 
 	/* Init a new repo */
 	cl_set_cleanup(&cleanup_repository, "not.overwrite.git");
-	cl_git_pass(git_repository_init(&_repo, "not.overwrite.git", 1));
+	cl_git_pass(git_repository_init(&g_repo, "not.overwrite.git", 1));
 
 	/* Change the "core.ignorecase" config value to something unlikely */
-	git_repository_config(&config, _repo);
+	git_repository_config(&config, g_repo);
 	git_config_set_int32(config, "core.ignorecase", 42);
 	git_config_free(config);
-	git_repository_free(_repo);
-	_repo = NULL;
+	git_repository_free(g_repo);
+	g_repo = NULL;
 
 	/* Reinit the repository */
-	cl_git_pass(git_repository_init(&_repo, "not.overwrite.git", 1));
-	git_repository_config(&config, _repo);
+	cl_git_pass(git_repository_init(&g_repo, "not.overwrite.git", 1));
+	git_repository_config(&config, g_repo);
 
 	/* Ensure the "core.ignorecase" config value hasn't been updated */
 	cl_git_pass(git_config_get_int32(&current_value, config, "core.ignorecase"));
@@ -280,26 +357,26 @@ void test_repo_init__reinit_overwrites_filemode(void)
 
 	/* Init a new repo */
 	cl_set_cleanup(&cleanup_repository, "overwrite.git");
-	cl_git_pass(git_repository_init(&_repo, "overwrite.git", 1));
+	cl_git_pass(git_repository_init(&g_repo, "overwrite.git", 1));
 
 	/* Change the "core.filemode" config value to something unlikely */
-	cl_repo_set_bool(_repo, "core.filemode", !expected);
+	cl_repo_set_bool(g_repo, "core.filemode", !expected);
 
-	git_repository_free(_repo);
-	_repo = NULL;
+	git_repository_free(g_repo);
+	g_repo = NULL;
 
 	/* Reinit the repository */
-	cl_git_pass(git_repository_init(&_repo, "overwrite.git", 1));
+	cl_git_pass(git_repository_init(&g_repo, "overwrite.git", 1));
 
 	/* Ensure the "core.filemode" config value has been reset */
-	current_value = cl_repo_get_bool(_repo, "core.filemode");
+	current_value = cl_repo_get_bool(g_repo, "core.filemode");
 	cl_assert_equal_i(expected, current_value);
 }
 
 void test_repo_init__sets_logAllRefUpdates_according_to_type_of_repository(void)
 {
 	assert_config_entry_on_init_bytype("core.logallrefupdates", GIT_ENOTFOUND, true);
-	git_repository_free(_repo);
+	git_repository_free(g_repo);
 	assert_config_entry_on_init_bytype("core.logallrefupdates", true, false);
 }
 
@@ -308,16 +385,16 @@ void test_repo_init__extended_0(void)
 	git_repository_init_options opts = GIT_REPOSITORY_INIT_OPTIONS_INIT;
 
 	/* without MKDIR this should fail */
-	cl_git_fail(git_repository_init_ext(&_repo, "extended", &opts));
+	cl_git_fail(git_repository_init_ext(&g_repo, "extended", &opts));
 
 	/* make the directory first, then it should succeed */
-	cl_git_pass(git_futils_mkdir("extended", NULL, 0775, 0));
-	cl_git_pass(git_repository_init_ext(&_repo, "extended", &opts));
+	cl_git_pass(git_futils_mkdir("extended", 0775, 0));
+	cl_git_pass(git_repository_init_ext(&g_repo, "extended", &opts));
 
-	cl_assert(!git__suffixcmp(git_repository_workdir(_repo), "/extended/"));
-	cl_assert(!git__suffixcmp(git_repository_path(_repo), "/extended/.git/"));
-	cl_assert(!git_repository_is_bare(_repo));
-	cl_assert(git_repository_is_empty(_repo));
+	cl_assert(!git__suffixcmp(git_repository_workdir(g_repo), "/extended/"));
+	cl_assert(!git__suffixcmp(git_repository_path(g_repo), "/extended/.git/"));
+	cl_assert(!git_repository_is_bare(g_repo));
+	cl_assert(git_repository_is_empty(g_repo));
 
 	cleanup_repository("extended");
 }
@@ -337,191 +414,103 @@ void test_repo_init__extended_1(void)
 	opts.initial_head = "development";
 	opts.origin_url = "https://github.com/libgit2/libgit2.git";
 
-	cl_git_pass(git_repository_init_ext(&_repo, "root/b/c.git", &opts));
+	cl_git_pass(git_repository_init_ext(&g_repo, "root/b/c.git", &opts));
 
-	cl_assert(!git__suffixcmp(git_repository_workdir(_repo), "/c_wd/"));
-	cl_assert(!git__suffixcmp(git_repository_path(_repo), "/c.git/"));
+	cl_assert(!git__suffixcmp(git_repository_workdir(g_repo), "/c_wd/"));
+	cl_assert(!git__suffixcmp(git_repository_path(g_repo), "/c.git/"));
 	cl_assert(git_path_isfile("root/b/c_wd/.git"));
-	cl_assert(!git_repository_is_bare(_repo));
+	cl_assert(!git_repository_is_bare(g_repo));
 	/* repo will not be counted as empty because we set head to "development" */
-	cl_assert(!git_repository_is_empty(_repo));
+	cl_assert(!git_repository_is_empty(g_repo));
 
-	cl_git_pass(git_path_lstat(git_repository_path(_repo), &st));
+	cl_git_pass(git_path_lstat(git_repository_path(g_repo), &st));
 	cl_assert(S_ISDIR(st.st_mode));
 	if (cl_is_chmod_supported())
 		cl_assert((S_ISGID & st.st_mode) == S_ISGID);
 	else
 		cl_assert((S_ISGID & st.st_mode) == 0);
 
-	cl_git_pass(git_reference_lookup(&ref, _repo, "HEAD"));
-	cl_assert(git_reference_type(ref) == GIT_REF_SYMBOLIC);
+	cl_git_pass(git_reference_lookup(&ref, g_repo, "HEAD"));
+	cl_assert(git_reference_type(ref) == GIT_REFERENCE_SYMBOLIC);
 	cl_assert_equal_s("refs/heads/development", git_reference_symbolic_target(ref));
 	git_reference_free(ref);
 
-	cl_git_pass(git_remote_load(&remote, _repo, "origin"));
+	cl_git_pass(git_remote_lookup(&remote, g_repo, "origin"));
 	cl_assert_equal_s("origin", git_remote_name(remote));
 	cl_assert_equal_s(opts.origin_url, git_remote_url(remote));
 	git_remote_free(remote);
 
-	git_repository_free(_repo);
+	git_repository_free(g_repo);
 	cl_fixture_cleanup("root");
 }
 
-#define CLEAR_FOR_CORE_FILEMODE(M) ((M) &= ~0177)
-
-static void assert_hooks_match(
-	const char *template_dir,
-	const char *repo_dir,
-	const char *hook_path,
-	bool core_filemode)
+void test_repo_init__relative_gitdir(void)
 {
-	git_buf expected = GIT_BUF_INIT;
-	git_buf actual = GIT_BUF_INIT;
-	struct stat expected_st, st;
-
-	cl_git_pass(git_buf_joinpath(&expected, template_dir, hook_path));
-	cl_git_pass(git_path_lstat(expected.ptr, &expected_st));
-
-	cl_git_pass(git_buf_joinpath(&actual, repo_dir, hook_path));
-	cl_git_pass(git_path_lstat(actual.ptr, &st));
-
-	cl_assert(expected_st.st_size == st.st_size);
-
-	if (GIT_MODE_TYPE(expected_st.st_mode) != GIT_FILEMODE_LINK) {
-		mode_t expected_mode =
-			GIT_MODE_TYPE(expected_st.st_mode) |
-			(GIT_PERMS_FOR_WRITE(expected_st.st_mode) & ~g_umask);
-
-		if (!core_filemode) {
-			CLEAR_FOR_CORE_FILEMODE(expected_mode);
-			CLEAR_FOR_CORE_FILEMODE(st.st_mode);
-		}
-
-		cl_assert_equal_i_fmt(expected_mode, st.st_mode, "%07o");
-	}
-
-	git_buf_free(&expected);
-	git_buf_free(&actual);
-}
-
-static void assert_mode_seems_okay(
-	const char *base, const char *path,
-	git_filemode_t expect_mode, bool expect_setgid, bool core_filemode)
-{
-	git_buf full = GIT_BUF_INIT;
-	struct stat st;
-
-	cl_git_pass(git_buf_joinpath(&full, base, path));
-	cl_git_pass(git_path_lstat(full.ptr, &st));
-	git_buf_free(&full);
-
-	if (!core_filemode) {
-		CLEAR_FOR_CORE_FILEMODE(expect_mode);
-		CLEAR_FOR_CORE_FILEMODE(st.st_mode);
-		expect_setgid = false;
-	}
-
-	if (S_ISGID != 0)
-		cl_assert_equal_b(expect_setgid, (st.st_mode & S_ISGID) != 0);
-
-	cl_assert_equal_b(
-		GIT_PERMS_IS_EXEC(expect_mode), GIT_PERMS_IS_EXEC(st.st_mode));
-
-	cl_assert_equal_i_fmt(
-		GIT_MODE_TYPE(expect_mode), GIT_MODE_TYPE(st.st_mode), "%07o");
-}
-
-void test_repo_init__extended_with_template(void)
-{
-	git_buf expected = GIT_BUF_INIT;
-	git_buf actual = GIT_BUF_INIT;
 	git_repository_init_options opts = GIT_REPOSITORY_INIT_OPTIONS_INIT;
-	int filemode;
+	git_buf dot_git_content = GIT_BUF_INIT;
 
-	cl_set_cleanup(&cleanup_repository, "templated.git");
+	opts.workdir_path = "../c_wd";
+	opts.flags =
+		GIT_REPOSITORY_INIT_MKPATH |
+		GIT_REPOSITORY_INIT_RELATIVE_GITLINK |
+		GIT_REPOSITORY_INIT_NO_DOTGIT_DIR;
 
-	opts.flags = GIT_REPOSITORY_INIT_MKPATH | GIT_REPOSITORY_INIT_BARE |
-		GIT_REPOSITORY_INIT_EXTERNAL_TEMPLATE;
-	opts.template_path = cl_fixture("template");
+	/* make the directory first, then it should succeed */
+	cl_git_pass(git_repository_init_ext(&g_repo, "root/b/my_repository", &opts));
 
-	cl_git_pass(git_repository_init_ext(&_repo, "templated.git", &opts));
+	cl_assert(!git__suffixcmp(git_repository_workdir(g_repo), "root/b/c_wd/"));
+	cl_assert(!git__suffixcmp(git_repository_path(g_repo), "root/b/my_repository/"));
+	cl_assert(!git_repository_is_bare(g_repo));
+	cl_assert(git_repository_is_empty(g_repo));
 
-	cl_assert(git_repository_is_bare(_repo));
+	/* Verify that the gitlink and worktree entries are relative */
 
-	cl_assert(!git__suffixcmp(git_repository_path(_repo), "/templated.git/"));
+	/* Verify worktree */
+	assert_config_entry_value(g_repo, "core.worktree", "../c_wd/");
 
-	cl_git_pass(git_futils_readbuffer(
-		&expected, cl_fixture("template/description")));
-	cl_git_pass(git_futils_readbuffer(
-		&actual, "templated.git/description"));
+	/* Verify gitlink */
+	cl_git_pass(git_futils_readbuffer(&dot_git_content, "root/b/c_wd/.git"));
+	cl_assert_equal_s("gitdir: ../my_repository/", dot_git_content.ptr);
 
-	cl_assert_equal_s(expected.ptr, actual.ptr);
-
-	git_buf_free(&expected);
-	git_buf_free(&actual);
-
-	filemode = cl_repo_get_bool(_repo, "core.filemode");
-
-	assert_hooks_match(
-		cl_fixture("template"), git_repository_path(_repo),
-		"hooks/update.sample", filemode);
-
-	assert_hooks_match(
-		cl_fixture("template"), git_repository_path(_repo),
-		"hooks/link.sample", filemode);
+	git_buf_dispose(&dot_git_content);
+	cleanup_repository("root");
 }
 
-void test_repo_init__extended_with_template_and_shared_mode(void)
+void test_repo_init__relative_gitdir_2(void)
 {
-	git_buf expected = GIT_BUF_INIT;
-	git_buf actual = GIT_BUF_INIT;
 	git_repository_init_options opts = GIT_REPOSITORY_INIT_OPTIONS_INIT;
-	int filemode = true;
-	const char *repo_path = NULL;
+	git_buf dot_git_content = GIT_BUF_INIT;
+	git_buf full_path = GIT_BUF_INIT;
 
-	cl_set_cleanup(&cleanup_repository, "init_shared_from_tpl");
+	cl_git_pass(git_path_prettify(&full_path, ".", NULL));
+	cl_git_pass(git_buf_joinpath(&full_path, full_path.ptr, "root/b/c_wd"));
 
-	opts.flags = GIT_REPOSITORY_INIT_MKPATH |
-		GIT_REPOSITORY_INIT_EXTERNAL_TEMPLATE;
-	opts.template_path = cl_fixture("template");
-	opts.mode = GIT_REPOSITORY_INIT_SHARED_GROUP;
+	opts.workdir_path = full_path.ptr;
+	opts.flags =
+		GIT_REPOSITORY_INIT_MKPATH |
+		GIT_REPOSITORY_INIT_RELATIVE_GITLINK |
+		GIT_REPOSITORY_INIT_NO_DOTGIT_DIR;
 
-	cl_git_pass(git_repository_init_ext(&_repo, "init_shared_from_tpl", &opts));
+	/* make the directory first, then it should succeed */
+	cl_git_pass(git_repository_init_ext(&g_repo, "root/b/my_repository", &opts));
+	git_buf_dispose(&full_path);
 
-	cl_assert(!git_repository_is_bare(_repo));
-	cl_assert(!git__suffixcmp(git_repository_path(_repo), "/init_shared_from_tpl/.git/"));
+	cl_assert(!git__suffixcmp(git_repository_workdir(g_repo), "root/b/c_wd/"));
+	cl_assert(!git__suffixcmp(git_repository_path(g_repo), "root/b/my_repository/"));
+	cl_assert(!git_repository_is_bare(g_repo));
+	cl_assert(git_repository_is_empty(g_repo));
 
-	filemode = cl_repo_get_bool(_repo, "core.filemode");
+	/* Verify that the gitlink and worktree entries are relative */
 
-	cl_git_pass(git_futils_readbuffer(
-		&expected, cl_fixture("template/description")));
-	cl_git_pass(git_futils_readbuffer(
-		&actual, "init_shared_from_tpl/.git/description"));
+	/* Verify worktree */
+	assert_config_entry_value(g_repo, "core.worktree", "../c_wd/");
 
-	cl_assert_equal_s(expected.ptr, actual.ptr);
+	/* Verify gitlink */
+	cl_git_pass(git_futils_readbuffer(&dot_git_content, "root/b/c_wd/.git"));
+	cl_assert_equal_s("gitdir: ../my_repository/", dot_git_content.ptr);
 
-	git_buf_free(&expected);
-	git_buf_free(&actual);
-
-	repo_path = git_repository_path(_repo);
-	assert_mode_seems_okay(repo_path, "hooks",
-		GIT_FILEMODE_TREE | GIT_REPOSITORY_INIT_SHARED_GROUP, true, filemode);
-	assert_mode_seems_okay(repo_path, "info",
-		GIT_FILEMODE_TREE | GIT_REPOSITORY_INIT_SHARED_GROUP, true, filemode);
-	assert_mode_seems_okay(repo_path, "description",
-		GIT_FILEMODE_BLOB, false, filemode);
-
-	/* for a non-symlinked hook, it should have shared permissions now */
-	assert_hooks_match(
-		cl_fixture("template"), git_repository_path(_repo),
-		"hooks/update.sample", filemode);
-
-	/* for a symlinked hook, the permissions still should match the
-	 * source link, not the GIT_REPOSITORY_INIT_SHARED_GROUP value
-	 */
-	assert_hooks_match(
-		cl_fixture("template"), git_repository_path(_repo),
-		"hooks/link.sample", filemode);
+	git_buf_dispose(&dot_git_content);
+	cleanup_repository("root");
 }
 
 void test_repo_init__can_reinit_an_initialized_repository(void)
@@ -530,12 +519,12 @@ void test_repo_init__can_reinit_an_initialized_repository(void)
 
 	cl_set_cleanup(&cleanup_repository, "extended");
 
-	cl_git_pass(git_futils_mkdir("extended", NULL, 0775, 0));
-	cl_git_pass(git_repository_init(&_repo, "extended", false));
+	cl_git_pass(git_futils_mkdir("extended", 0775, 0));
+	cl_git_pass(git_repository_init(&g_repo, "extended", false));
 
 	cl_git_pass(git_repository_init(&reinit, "extended", false));
 
-	cl_assert_equal_s(git_repository_path(_repo), git_repository_path(reinit));
+	cl_assert_equal_s(git_repository_path(g_repo), git_repository_path(reinit));
 
 	git_repository_free(reinit);
 }
@@ -547,10 +536,10 @@ void test_repo_init__init_with_initial_commit(void)
 	cl_set_cleanup(&cleanup_repository, "committed");
 
 	/* Initialize the repository */
-	cl_git_pass(git_repository_init(&_repo, "committed", 0));
+	cl_git_pass(git_repository_init(&g_repo, "committed", 0));
 
-	/* Init will be automatically created when requested for a new repo */
-	cl_git_pass(git_repository_index(&index, _repo));
+	/* Index will be automatically created when requested for a new repo */
+	cl_git_pass(git_repository_index(&index, g_repo));
 
 	/* Create a file so we can commit it
 	 *
@@ -576,7 +565,7 @@ void test_repo_init__init_with_initial_commit(void)
 	/* Make sure we're ready to use git_signature_default :-) */
 	{
 		git_config *cfg, *local;
-		cl_git_pass(git_repository_config(&cfg, _repo));
+		cl_git_pass(git_repository_config(&cfg, g_repo));
 		cl_git_pass(git_config_open_level(&local, cfg, GIT_CONFIG_LEVEL_LOCAL));
 		cl_git_pass(git_config_set_string(local, "user.name", "Test User"));
 		cl_git_pass(git_config_set_string(local, "user.email", "t@example.com"));
@@ -590,12 +579,12 @@ void test_repo_init__init_with_initial_commit(void)
 		git_oid tree_id, commit_id;
 		git_tree *tree;
 
-		cl_git_pass(git_signature_default(&sig, _repo));
+		cl_git_pass(git_signature_default(&sig, g_repo));
 		cl_git_pass(git_index_write_tree(&tree_id, index));
-		cl_git_pass(git_tree_lookup(&tree, _repo, &tree_id));
+		cl_git_pass(git_tree_lookup(&tree, g_repo, &tree_id));
 
 		cl_git_pass(git_commit_create_v(
-			&commit_id, _repo, "HEAD", sig, sig,
+			&commit_id, g_repo, "HEAD", sig, sig,
 			NULL, "First", tree, 0));
 
 		git_tree_free(tree);
@@ -604,3 +593,98 @@ void test_repo_init__init_with_initial_commit(void)
 
 	git_index_free(index);
 }
+
+void test_repo_init__at_filesystem_root(void)
+{
+	git_repository *repo;
+	const char *sandbox = clar_sandbox_path();
+	git_buf root = GIT_BUF_INIT;
+	int root_len;
+
+	if (!cl_is_env_set("GITTEST_INVASIVE_FS_STRUCTURE"))
+		cl_skip();
+
+	root_len = git_path_root(sandbox);
+	cl_assert(root_len >= 0);
+
+	git_buf_put(&root, sandbox, root_len+1);
+	git_buf_joinpath(&root, root.ptr, "libgit2_test_dir");
+
+	cl_assert(!git_path_exists(root.ptr));
+
+	cl_git_pass(git_repository_init(&repo, root.ptr, 0));
+	cl_assert(git_path_isdir(root.ptr));
+	cl_git_pass(git_futils_rmdir_r(root.ptr, NULL, GIT_RMDIR_REMOVE_FILES));
+
+	git_buf_dispose(&root);
+	git_repository_free(repo);
+}
+
+void test_repo_init__nonexisting_directory(void)
+{
+	git_repository_init_options opts = GIT_REPOSITORY_INIT_OPTIONS_INIT;
+	git_repository *repo;
+
+	/*
+	 * If creating a repo with non-existing parent directories, then libgit2
+	 * will by default create the complete directory hierarchy if using
+	 * `git_repository_init`. Thus, let's use the extended version and not
+	 * set the `GIT_REPOSITORY_INIT_MKPATH` flag.
+	 */
+	cl_git_fail(git_repository_init_ext(&repo, "nonexisting/path", &opts));
+}
+
+void test_repo_init__nonexisting_root(void)
+{
+#ifdef GIT_WIN32
+	git_repository *repo;
+
+	/*
+	 * This really only depends on the nonexistence of the Q: drive. We
+	 * cannot implement the equivalent test on Unix systems, as there is
+	 * fundamentally no path that is disconnected from the root directory.
+	 */
+	cl_git_fail(git_repository_init(&repo, "Q:/non/existent/path", 0));
+	cl_git_fail(git_repository_init(&repo, "Q:\\non\\existent\\path", 0));
+#else
+	clar__skip();
+#endif
+}
+
+void test_repo_init__unwriteable_directory(void)
+{
+#ifndef GIT_WIN32
+	git_repository *repo;
+
+	if (geteuid() == 0)
+		clar__skip();
+
+	/*
+	 * Create a non-writeable directory so that we cannot create directories
+	 * inside of it. The root user has CAP_DAC_OVERRIDE, so he doesn't care
+	 * for the directory permissions and thus we need to skip the test if
+	 * run as root user.
+	 */
+	cl_must_pass(p_mkdir("unwriteable", 0444));
+	cl_git_fail(git_repository_init(&repo, "unwriteable/repo", 0));
+	cl_must_pass(p_rmdir("unwriteable"));
+#else
+	clar__skip();
+#endif
+}
+
+void test_repo_init__defaultbranch_config(void)
+{
+	git_reference *head;
+
+	cl_set_cleanup(&cleanup_repository, "repo");
+
+	create_tmp_global_config("tmp_global_path", "init.defaultbranch", "my_default_branch");
+
+	cl_git_pass(git_repository_init(&g_repo, "repo", 0));
+	cl_git_pass(git_reference_lookup(&head, g_repo, "HEAD"));
+
+	cl_assert_equal_s("refs/heads/my_default_branch", git_reference_symbolic_target(head));
+
+	git_reference_free(head);
+}
