@@ -22,8 +22,9 @@
 #include "lslutils/misc.h"
 #include "lslutils/globalsmanager.h"
 #include "lslutils/thread.h"
+#include "FileSystem/FileSystem.h"
 
-#ifndef WIN32
+#ifndef _WIN32
 #include <sys/stat.h>
 #endif
 
@@ -399,6 +400,7 @@ GameOptions Unitsync::GetMapOptions(const std::string& name)
 	}
 
 	if (MapExists(name)) {
+		GetMapHash(name);
 		const std::string filename = GetMapOptionsPath(name);
 		if (!lslcache.Get(filename, ret)) {
 			PrefetchMap(name);
@@ -734,7 +736,7 @@ std::string Unitsync::GetMac()
 std::string Unitsync::GetFileCachePath(const std::string& name, bool IsMod, bool usehash) const
 {
 	assert(!name.empty());
-	std::string ret = m_cache_path + name;
+	std::string ret = m_cache_path + CFileSystem::EscapeFilename(name);
 	if (!usehash)
 		return ret;
 
