@@ -4,6 +4,7 @@
 #include "Logger.h"
 #include "lib/md5/md5.h"
 #include "lib/base64/base64.h"
+#include "lsl/lslutils/platform.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -18,7 +19,8 @@ void SetDownloadListener(IDownloaderProcessUpdateListener listener)
 
 bool isEngineDownload(DownloadEnum::Category cat)
 {
-	return (cat == DownloadEnum::CAT_ENGINE_LINUX) ||
+	return (cat == DownloadEnum::CAT_ENGINE) ||
+	       (cat == DownloadEnum::CAT_ENGINE_LINUX) ||
 	       (cat == DownloadEnum::CAT_ENGINE_LINUX64) ||
 	       (cat == DownloadEnum::CAT_ENGINE_MACOSX) ||
 	       (cat == DownloadEnum::CAT_ENGINE_WINDOWS) ||
@@ -27,17 +29,17 @@ bool isEngineDownload(DownloadEnum::Category cat)
 
 DownloadEnum::Category getPlatformEngineCat()
 {
-#ifdef _WIN64
-	return DownloadEnum::CAT_ENGINE_WINDOWS64;
-#elif defined WIN32
-	return DownloadEnum::CAT_ENGINE_WINDOWS;
-#elif defined(__APPLE__)
-	return DownloadEnum::CAT_ENGINE_MACOSX;
-#elif defined(__x86_64__) || defined(__e2k__)
+	switch (LSL::Util::GetPlatform()){
+		case LSL::Util::Platform::kLinux32:   return DownloadEnum::CAT_ENGINE_LINUX;
+		case LSL::Util::Platform::kLinux64:   return DownloadEnum::CAT_ENGINE_LINUX64;
+		case LSL::Util::Platform::kWindows32: return DownloadEnum::CAT_ENGINE_WINDOWS;
+		case LSL::Util::Platform::kWindows64: return DownloadEnum::CAT_ENGINE_WINDOWS64;
+		case LSL::Util::Platform::kMacosx:    return DownloadEnum::CAT_ENGINE_MACOSX;
+		default:
+			assert(false);
+	}
+	assert(false);
 	return DownloadEnum::CAT_ENGINE_LINUX64;
-#else
-	return DownloadEnum::CAT_ENGINE_LINUX;
-#endif
 }
 
 bool download_engine(std::list<IDownload*>& dllist)
@@ -55,7 +57,7 @@ bool download_engine(std::list<IDownload*>& dllist)
 
 	httpDownload->download(enginedls);
 	for (const IDownload* dl : enginedls) {
-		if (!fileSystem->extractEngine(dl->name, dl->version)) {
+		if (!fileSystem->extractEngine(dl->name, dl->version, LSL::Util::GetCurrentPlatformString())) {
 			LOG_ERROR("Failed to extract engine %s", dl->version.c_str());
 			res = false;
 		}
@@ -96,6 +98,7 @@ bool search(DownloadEnum::Category cat, const char* name,
 	std::string searchname = name;
 
 	if (cat == DownloadEnum::CAT_ENGINE) {
+		// no platform specified, "translate" to current platform
 		cat = getPlatformEngineCat();
 	}
 
@@ -273,7 +276,6 @@ int DownloadStart()
 	rapidDownload->download(dls);
 	httpDownload->download(dls, 1);
 	download_engine(dls);
-	IDownloader::freeResult(searchres);
 	int res = 0;
 	for (const IDownload* dl: dls) {
 		if (dl->state != IDownload::STATE_FINISHED) {
