@@ -1,18 +1,17 @@
 /* This file is part of the Springlobby (GPL v2 or later), see COPYING */
-
 #ifndef SPRINGLOBBY_LIBNOTIFY_H
 #define SPRINGLOBBY_LIBNOTIFY_H
 
 #ifdef HAVE_LIBNOTIFY
-
 #include "libnotify.h"
+
+#include <libnotify/notify.h>
+#include <wx/bitmap.h>
+#include <wx/log.h>
+
 #include "settings.h"
 #include "utils/version.h"
 
-#include <wx/log.h>
-#include <libnotify/notify.h>
-
-
 LibnotifyNotification::LibnotifyNotification(wxWindow*)
 {
 }
@@ -25,9 +24,9 @@ void LibnotifyNotification::Show(const wxBitmap& icon, const size_t /*pos*/, con
 	NotifyNotification* n;
 	notify_init("Test");
 #if !defined(NOTIFY_VERSION_MINOR) || (NOTIFY_VERSION_MAJOR == 0 && NOTIFY_VERSION_MINOR < 7)
-	n = notify_notification_new(getSpringlobbyName().c_str(), data.second.mb_str(), NULL, NULL);
+	n = notify_notification_new(GetSpringlobbyName().c_str(), data.second.mb_str(), NULL, NULL);
 #else
-	n = notify_notification_new(getSpringlobbyName().c_str(), data.second.mb_str(), NULL);
+	n = notify_notification_new(GetSpringlobbyName().c_str(), data.second.mb_str(), NULL);
 #endif
 	notify_notification_set_timeout(n, sett().GetNotificationPopupDisplayTime() * 1000);
 
