@@ -4,7 +4,6 @@
 #define SPRINGLOBBY_HEADER_GUARD_TORRENTOPTIONSPANEL_H_INCLUDED
 
 #include <wx/scrolwin.h>
-
 class wxCheckBox;
 class wxTextCtrl;
 class Ui;
@@ -14,20 +13,25 @@ class wxButton;
 class wxStaticText;
 class wxSpinCtrl;
 class wxStaticBoxSizer;
-class TorrentOptionsPanel : public wxScrolledWindow
+class DownloadOptionsPanel : public wxScrolledWindow
 {
 public:
-	TorrentOptionsPanel(wxWindow* parent);
-	~TorrentOptionsPanel();
+	DownloadOptionsPanel(wxWindow* parent);
+	~DownloadOptionsPanel();
 
 	void OnApply(wxCommandEvent& event);
 	void OnRestore(wxCommandEvent& event);
+	void OnNewDirectory(wxCommandEvent& event);
+
+private:
+	void EnableSettings(bool enable);
+	void SetStatusDisplay();
 
 private:
 	wxSpinCtrl* m_parallel_http;
 	wxStaticBoxSizer* m_parallel_http_sizer;
-	void EnableSettings(bool enable);
-	void SetStatusDisplay();
+	wxButton* m_NewDownloadDirButton;
+	wxTextCtrl* m_DownloadDirectoryTextCtrl;
 
 	enum {
 		ID_ENABLEP2P = wxID_HIGHEST,
@@ -39,7 +43,8 @@ private:
 		ID_RESTORE,
 		ID_GAMESTART_RADIO,
 		ID_INGAME_UP,
-		ID_INGAME_DOWN
+		ID_INGAME_DOWN,
+		ID_SELECT_NEW_DIRECTORY
 	};
 
 	DECLARE_EVENT_TABLE()
