@@ -1,20 +1,19 @@
 /* This file is part of the Springlobby (GPL v2 or later), see COPYING */
-
 #include "pastedialog.h"
 
-#include <wx/icon.h>
-#include <wx/string.h>
-#include <wx/frame.h>
-#include <wx/defs.h>
-#include <wx/sizer.h>
-#include <wx/button.h>
-#include <wx/stattext.h>
 #include <wx/artprov.h>
-#include <wx/statbmp.h>
+#include <wx/button.h>
+#include <wx/defs.h>
+#include <wx/frame.h>
+#include <wx/icon.h>
 #include <wx/listctrl.h>
+#include <wx/sizer.h>
+#include <wx/statbmp.h>
+#include <wx/stattext.h>
+#include <wx/string.h>
 
+#include "gui/iconscollection.h"
 #include "utils/conversion.h"
-#include "iconimagelist.h"
 
 //in < 29 this is defined in wxDialogBase, which seems to have disappeared
 enum { ButtonSizerFlags = wxOK | wxCANCEL | wxYES | wxNO | wxHELP | wxNO_DEFAULT };
@@ -27,7 +26,9 @@ END_EVENT_TABLE()
 PasteDialog::PasteDialog(wxWindow* parent, const wxString& message)
     : wxDialog(parent, -1, _("Flood warning"), wxDefaultPosition, wxDefaultSize, wxFRAME_FLOAT_ON_PARENT | wxDEFAULT_DIALOG_STYLE)
 {
-	SetIcon(icons().GetIcon(icons().ICON_SPRINGLOBBY));
+	wxIcon mainIcon = wxIcon();
+	mainIcon.CopyFromBitmap(IconsCollection::Instance()->BMP_SPRINGLOBBY);
+	SetIcon(mainIcon);
 
 	//******** copied from wxsource/generic/msgdlgg.cpp with small modifications***********************************************************
 	m_main_sizer = new wxBoxSizer(wxVERTICAL);
@@ -46,7 +47,7 @@ PasteDialog::PasteDialog(wxWindow* parent, const wxString& message)
 	m_main_sizer->Add(0, 10);
 
 	// 3) buttons
-	wxSizer* sizerBtn = CreateButtonSizer(wxYES_NO);
+	wxSizer* sizerBtn = CreateButtonSizer(wxYES_NO | wxCANCEL);
 	if (sizerBtn) {
 		wxButton* but = new wxButton(this, ID_PASTE_BUTTON, _("Use pastebin"));
 		sizerBtn->Add(but, 0, wxALL, 10);
