@@ -59,7 +59,15 @@
 #endif
 
 
-SLCONFIG("/General/AutoUpdate", true, "Determines if springlobby should check for updates on startup");
+#if AUTOUPDATE
+#define AUTOUPDATE_DEFAULT true
+#else
+#define AUTOUPDATE_DEFAULT false
+#endif
+
+SLCONFIG("/General/AutoUpdate", AUTOUPDATE_DEFAULT, "Determines if springlobby should check for updates on startup");
+
+
 SLCONFIG("/GUI/StartTab", (long)MainWindow::PAGE_MULTIPLAYER, "which tab to show on startup");
 SLCONFIG("/Chat/BroadcastEverywhere", true, "setting to spam the server messages in all channels");
 SLCONFIG("/Server/Autoconnect", true, "Connect to server on startup");
@@ -787,7 +795,7 @@ void Ui::OnLobbyDownloaded(wxCommandEvent& data)
 	}
 	const wxString m_newexe = TowxString(SlPaths::GetUpdateDir()) + _T("springlobby_updater.exe");
 	wxArrayString params;
-#ifdef WIN32
+#ifdef _WIN32
 	DWORD id = GetCurrentProcessId();
 #else
 	long id = wxThread::GetCurrentId();
