@@ -24,6 +24,21 @@ lsl/battle/ibattle.h
 const unsigned int DEFAULT_SERVER_PORT = 8452;
 const unsigned int DEFAULT_EXTERNAL_UDP_SOURCE_PORT = 16941;
 
+const std::string GAME_BL_COLOURS[11] = {
+	"#ffffff",
+	"#ffffdd",
+	"#ddffdd",
+	"#fff2ee",
+	"#ffdddd",
+	"#eeeeee",
+	"#eeffff",
+	"#ddeedd",
+	"#fff2dd",
+	"#f3f3f3",
+	"#eef2ff"
+};
+
+
 class IBattle;
 class IServer;
 class AutoHost;
@@ -228,6 +243,8 @@ public:
 	virtual void SetLocalGame(const LSL::UnitsyncGame& game);
 	virtual const LSL::UnitsyncGame& LoadGame();
 	virtual const std::string& GetHostGameName() const;
+	virtual const std::string& GetHostGameNameAndVersion() const;
+	virtual const std::string& GetHostGameBackgroundColour() const;
 	virtual const std::string& GetHostGameHash() const;
 
 	virtual bool MapExists(bool comparehash = true) const;
@@ -719,6 +736,8 @@ private:
 	LSL::UnitsyncMap m_host_map;
 	LSL::UnitsyncGame m_host_game;
 	LSL::UnitsyncGame m_local_game;
+	std::string gameNameWithoutVersion;
+	std::string gameBackgroundColour;
 
 	std::map<std::string, int> m_restricted_units;
 	std::string m_previous_local_game_name;
