@@ -3,7 +3,6 @@
 #ifndef SPRINGLOBBY_REPLAYLIST_H_INCLUDED
 #define SPRINGLOBBY_REPLAYLIST_H_INCLUDED
 
-
 #include <wx/string.h>
 
 #include "iplaybacklist.h"
@@ -34,20 +33,23 @@ struct DemoFileHeader {
 */
 
 struct StoredGame;
-class wxFile;
+class PlayBackDataReader;
+class wxInputStream;
 
 class ReplayList : public IPlaybackList
 {
 public:
-	virtual void LoadPlaybacks(const std::vector<std::string>& filenames);
 	ReplayList();
 
 private:
-	bool GetReplayInfos(const std::string& ReplayPath, StoredGame& ret) const;
-	int replayVersion(wxFile& ReplayPath) const;
-	std::string GetScriptFromReplay(wxFile& ReplayPath, const int version) const;
+	bool GetReplayInfos(const std::string& ReplayPath, StoredGame& ret) const override;
+	int replayVersion(PlayBackDataReader& replay) const;
+
+	std::string GetEngineVersionFromReplay(PlayBackDataReader& replay, const int version) const;
+	std::string GetScriptFromReplay(PlayBackDataReader& replay, const int version) const;
+	uint64_t GetStartTimeStampFromReplay(PlayBackDataReader& replay, const int version) const;
 	//! saves relevant infos from header into replay struct
-	void GetHeaderInfo(wxFile& ReplayPath, StoredGame& rep, const int version) const;
+	void GetHeaderInfo(PlayBackDataReader& replay, StoredGame& rep, const int version) const;
 };
 
 IPlaybackList& replaylist();
