@@ -11,7 +11,7 @@ lsl/networking/socket.cpp
 **/
 #include "socket.h"
 
-#ifdef WIN32
+#ifdef _WIN32
 #ifndef NOMINMAX
 #define NOMINMAX
 #endif // NOMINMAX
