@@ -107,7 +107,7 @@ bool TASServer::ExecuteSayCommand(const std::string& cmdstr) //FIXME: all the /c
 		switch (arrayparams.size()) {
 			case 3:
 				password = arrayparams[2];
-			// no break wanted, next assignement is required
+				[[fallthrough]]; // no break wanted, next assignement is required
 			case 2:
 				channel = arrayparams[1];
 				break;
@@ -491,7 +491,8 @@ void TASServer::ExecuteCommand(const std::string& in)
 		return;
 	try {
 		ASSERT_LOGIC(params.AfterFirst('\n').IsEmpty(), "losing data");
-	} catch (...) {
+	} catch (const std::exception& e) {
+		wxLogWarning(_T("Exception: %s"), e.what());
 		return;
 	}
 	if (params[0] == '#') {
@@ -504,8 +505,10 @@ void TASServer::ExecuteCommand(const std::string& in)
 
 	try {
 		ExecuteCommand(STD_STRING(cmd), STD_STRING(params), replyid);
-	} catch (...) { // catch everything so the app doesn't crash, may makes odd beahviours but it's better than crashing randomly for normal users
-		wxLogWarning("Exception in ExecuteCommand");
+	} catch (const std::exception& e) {
+		// catch everything so the app doesn't crash, may make SL behave oddly,
+		// but it's better than crashing randomly for normal users.
+		wxLogWarning(_T("Exception: %s %s %s"), e.what(), cmd, params);
 	}
 }
 
@@ -549,7 +552,9 @@ void TASServer::ExecuteCommand(const std::string& cmd, const std::string& inpara
 			m_server_lanmode = GetBoolParam(params);
 
 			if (m_do_register) {
-				SendCmd("REGISTER", m_serverinfo.username + std::string(" ") + GetPasswordHash(m_serverinfo.password) + std::string(" ") + m_serverinfo.email);
+				SendCmd("REGISTER", m_serverinfo.username
+				  + std::string(" ") + GetPasswordHash(m_serverinfo.password)
+				  + std::string(" ") + m_serverinfo.email);
 			} else {
 				m_se->OnConnected(m_serverinfo.description, "", true, m_supported_spring_version, m_server_lanmode);
 			}
@@ -719,7 +724,8 @@ void TASServer::ExecuteCommand(const std::string& cmd, const std::string& inpara
 							return;
 					}
 					SetRelayIngamePassword(usr);
-				} catch (...) {
+				} catch (const std::exception& e) {
+					wxLogWarning(_T("Exception: %s"), e.what());
 				}
 				return;
 			}
@@ -758,7 +764,8 @@ void TASServer::ExecuteCommand(const std::string& cmd, const std::string& inpara
 		try {
 			if (GetBattle(id).IsProxy())
 				RelayCmd("SUPPORTSCRIPTPASSWORD"); // send flag to relayhost marking we support script passwords
-		} catch (...) {
+		} catch (const std::exception& e) {
+			wxLogWarning(_T("Exception: %s"), e.what());
 		}
 	} else if (cmd == "CLIENTBATTLESTATUS") {
 		nick = GetWordParam(params);
@@ -1200,7 +1207,8 @@ void TASServer::Ring(const std::string& nick)
 
 		SendCmd("RING", nick, battle.IsProxy());
 
-	} catch (...) {
+	} catch (const std::exception& e) {
+		wxLogWarning(_T("Exception: %s"), e.what());
 		SendCmd("RING", nick);
 	}
 }
@@ -1426,7 +1434,8 @@ void TASServer::SendHostInfo(HostInfo update)
 	IBattle& battle = GetBattle(m_battle_id);
 	try {
 		ASSERT_LOGIC(battle.IsFounderMe(), "I'm not founder");
-	} catch (...) {
+	} catch (const std::exception& e) {
+		wxLogWarning(_T("Exception: %s"), e.what());
 		return;
 	}
 
@@ -1517,7 +1526,8 @@ void TASServer::SendHostInfo(const std::string& Tag)
 
 	try {
 		ASSERT_LOGIC(battle.IsFounderMe(), "I'm not founder");
-	} catch (...) {
+	} catch (const std::exception& e) {
+		wxLogWarning(_T("Exception: %s"), e.what());
 		return;
 	}
 
@@ -1556,7 +1566,8 @@ void TASServer::SendUserPosition(const User& user)
 		std::string msgy = stdprintf("game/Team%d/StartPosY=%d", status.team, status.pos.y);
 		std::string netmessage = msgx + "\t" + msgy;
 		SendCmd("SETSCRIPTTAGS", netmessage, battle.IsProxy());
-	} catch (...) {
+	} catch (const std::exception& e) {
+		wxLogWarning(_T("Exception: %s"), e.what());
 		return;
 	}
 }
@@ -1572,7 +1583,8 @@ IBattle* TASServer::GetCurrentBattle()
 	try {
 		ASSERT_EXCEPTION(m_battle_id != -1, _T("invalid m_battle_id value"));
 		ASSERT_LOGIC(BattleExists(m_battle_id), "battle doesn't exists");
-	} catch (...) {
+	} catch (const std::exception& e) {
+		wxLogWarning(_T("Exception: %s"), e.what());
 		return NULL;
 	}
 
@@ -2024,7 +2036,8 @@ LSL::StringVector TASServer::GetRelayHostList()
 			if (manager.Status().in_game || manager.Status().away)
 				continue; // skip the manager is not connected or reports it's ingame ( no slots available ), or it's away ( functionality disabled )
 			ret.push_back(m_relay_host_manager_list[i]);
-		} catch (...) {
+		} catch (const std::exception& e) {
+			wxLogWarning(_T("Exception: %s"), e.what());
 		}
 	}
 	return ret;
