@@ -1,9 +1,10 @@
 /* This file is part of the Springlobby (GPL v2 or later), see COPYING */
-
 #define BOOST_TEST_MODULE lobbyid
-#include <boost/test/unit_test.hpp>
 
+#include <boost/test/unit_test.hpp>
 #include <stdio.h>
+
+#include "address.h"
 #include "utils/crc.h"
 
 BOOST_AUTO_TEST_CASE(lobbyid)
@@ -39,3 +40,11 @@ BOOST_AUTO_TEST_CASE(lobbyid)
 	m_crc.UpdateData("The quick brown fox jumps over the lazy dog");
 	BOOST_CHECK(m_crc.GetCRC() == 1095738169); // == 414fa339, http://rosettacode.org/wiki/CRC-32#C.2B.2B
 }
+
+BOOST_AUTO_TEST_CASE(getmac)
+{
+	std::vector<unsigned char> mac;
+	GetMac(mac);
+	const std::string smac = MacToString(mac);
+	printf("%s\n", smac.c_str());
+}
