@@ -5,10 +5,11 @@
 
 class wxColour;
 class wxArrayString;
-namespace LSL{
-	class lslColor;
-	typedef std::vector< std::string > StringVector;
-}
+namespace LSL
+{
+class lslColor;
+typedef std::vector<std::string> StringVector;
+} // namespace LSL
 
 wxColour lslTowxColour(const LSL::lslColor& col);
 LSL::lslColor wxColourTolsl(const wxColour& col);
