@@ -3,9 +3,8 @@
 #ifndef SPRINGLOBBY_HEADERGUARD_PLATFORM_H
 #define SPRINGLOBBY_HEADERGUARD_PLATFORM_H
 
-#include <wx/string.h>
 #include <wx/arrstr.h>
-
+#include <wx/string.h>
 /**
     let origin be /path/to/some/dir and destination /some/other/path
     this will copy dir (and everything below that recursively to /some/other/path/dir
@@ -17,7 +16,7 @@ bool MoveDirWithFilebackupRename(wxString from, wxString to, bool backup = true,
 class CwdGuard
 {
 public:
-	CwdGuard(const wxString& new_cwd);
+	explicit CwdGuard(const wxString& new_cwd);
 	~CwdGuard();
 
 private:
@@ -28,7 +27,6 @@ int RunProcess(const wxString& cmd, const wxArrayString& params, const bool asyn
 int BrowseFolder(const wxString& path);
 int WaitForExit(int pid);
 void ErrorMsgBox(const wxString& err, bool silent = false);
-bool SafeMkdir(const wxString& dir);
 
 //!
 
